/*=========================================================================
This file is part of Broken Soul.

    Broken Soul is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Broken Soul is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Broken Soul.  If not, see <http://www.gnu.org/licenses/>.
=========================================================================*/

#include <PA9.h>
#include "include/colisao.hpp"
#include <string.h>

/* Pega o tile que esta na posicao x, y */
u8 getTile(s16 x, s16 y, const char* referencia)
{
	if(strcmp(referencia, "mapa") == 0)
	{
		return mapaMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "cidade") == 0)
	{
		return cidadeMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "castelo") == 0)
	{
		return casteloMap_Map[((y>>3)*32) + (x>>3)];
	}
	else if(strcmp(referencia, "inn") == 0)
	{
		return innMap_Map[((y>>3)*32) + (x>>3)];
	}
	else if(strcmp(referencia, "floresta1") == 0)
	{
		return floresta1Map_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "floresta2") == 0)
	{
		return floresta2Map_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "deserto") == 0)
	{
		return desertoMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "deserto2") == 0)
	{
		return deserto2Map_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "cenarioCaverna") == 0)
	{
		return cenarioCavernaMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "cenarioCaverna2") == 0)
	{
		return cenarioCaverna2Map_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "cenarioCaverna3") == 0)
	{
		return cenarioCaverna3Map_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "Rocha") == 0)
	{
		return RochaMap_Map[((y>>3)*32) + (x>>3)];
	}
	else if(strcmp(referencia, "batalhaDeserto") == 0)
	{
		return batalhaDesertoMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "batalhaCaverna") == 0)
	{
		return batalhaCavernaMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "batalhaRocha") == 0)
	{
		return batalhaRochaMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "batalhaFloresta") == 0)
	{
		return batalhaFlorestaMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "batalhaMapa") == 0)
	{
		return batalhaMapaMap_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "vulcao1") == 0)
	{
		PA_OutputText(0 ,5, 3, "vulcao1");
		return vulcao1Map_Map[((y>>3)*64) + (x>>3)];
	}
	else if(strcmp(referencia, "vulcao2") == 0)
	{
		return vulcao2Map_Map[((y>>3)*64) + (x>>3)];
	}
	else
	{
		return -1;
	}

	//y>>3 = tamanho do tile, os tiles sao de tamanaho 8x8 pixels
	// *128, quantos tile exitem ao longo do eixo x, no nosso caso, este sprite tem 1024 pixels e como cada tile tem 8x8
	// pixels, logo, temos num total de 1024%8 = 128 tiles
}

/* Pega os pixels em área */
u8 colisaoArea(s32 x, s32 y, s32 distanciaX, s32 distanciaY, const char* referencia)
{
	return getTile(x + distanciaX, y + distanciaY, referencia);
}

u8 tipoTile(u8 tile)
{
	switch (tile) {
		case 0:
			return FALSO;
		case 1:
			return VERDADEIRO;
		case 2:
			return CIDADE;
		case 3:
			return CAVERNA1;
		case 4:
			return FLORESTA1;
		case 5:
			return DESERTO1;
		case 6:
			return ROCHA;
		case 7:
			return VULCAO1;
		default:
			return FALSO;
	}
}



u8 maiorTile(u8 colisaoN, u8 colisaoL, u8 colisaoS, u8 colisaoO)
{
	u8 destino;
	destino = 0;
	if (colisaoN > destino)
		destino = colisaoN;
	
	if (colisaoL > destino)
		destino = colisaoL;
	
	if (colisaoS > destino)
		destino = colisaoS;
	
	if (colisaoO > destino)
		destino = colisaoO;
	
	return destino;
}

u8 colisaoHeroi(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 colisaoN, colisaoS, colisaoL, colisaoO;

	colisaoN = colisaoArea(posicaoX, posicaoY, 0, 6, referencia);
	colisaoL = colisaoArea(posicaoX, posicaoY, 8, 9, referencia);
	colisaoS = colisaoArea(posicaoX, posicaoY, 0, 11, referencia);
	colisaoO = colisaoArea(posicaoX, posicaoY, -8, 9, referencia);

	if(colisaoN == 1 || colisaoS == 1)
		*colisaoV = 1;

	if(colisaoL == 1 || colisaoO == 1)
		*colisaoH = 1;

	return maiorTile(colisaoN, colisaoL, colisaoS, colisaoO);
}

u8 colisaoMapa(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 tile;
	tile = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	return tipoTile(tile);
}

u8 colisaoCidade(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return INN;
	else if (destino == 3)
		return CASTELO;

	return MAPA;
}

u8 colisaoCastelo(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);

	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;

	return CIDADE;
}

u8 colisaoFloresta1(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return FLORESTA2;

	return MAPA;
}

u8 colisaoFloresta2(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);		
	if (destino == 0)
		return FALSO;	
	else if (destino == 1)
		return VERDADEIRO;
	
	return FLORESTA1;
}

u8 colisaoDeserto(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
			
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return DESERTO2;
		
	return MAPA;
}

u8 colisaoDeserto2(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
		
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return DESERTO1;
	else if (destino == 3)
		return FOSSIL;
	
	return MAPA;
}

u8 colisaoCaverna1(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
			
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return CAVERNA2;
	else if (destino == 3)
		return CAVERNA3;
	
	return MAPA;
}

u8 colisaoCaverna2(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return CHEFE1;

	return CAVERNA1;
}

u8 colisaoCaverna3(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return CHEFE2;
	
	return CAVERNA1;
}

u8 colisaoRocha(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
			
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	
	return MAPA;
}

u8 colisaoVulcao1(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
			
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return MAPA;
	
	return VULCAO2;
}

u8 colisaoVulcao2(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
			
	if (destino == 0)
		return FALSO;
	else if (destino == 1)
		return VERDADEIRO;
	else if (destino == 2)
		return VULCAO1;
	
	return CHEFE3;
}

u8 colisaoBatalha(s32 posicaoX, s32 posicaoY, const char *referencia, u8 *colisaoH, u8 *colisaoV)
{
	u8 destino;
	destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
			
	if (destino == 0)
		return FALSO;

	return VERDADEIRO;
}

/* Verifica se colide com o cenario */
u8 temColisao(s32 posicaoX, s32 posicaoY, u16 angulo, const char* referencia, u8 *colisaoH, u8 *colisaoV) //vou melhorar esta funcao ainda....
{
	/* Indica se o heroi vai mudar de cenario e para qual cenario vai. */
	u8 destino = 0;

	//----------------------------------------
	posicaoX += PA_Cos(angulo);				//
	posicaoY -= PA_Sin(angulo);				//
											//=> Esta parte quando colocar os inimigos talvez saia
	posicaoX = fix_norm(posicaoX);			//
	posicaoY = fix_norm(posicaoY);			//
	//---------------------------------------

	if (strcmp(referencia, "mapa") == 0) {
		return colisaoMapa(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	} 
	else if (strcmp(referencia, "cidade") == 0){
		return colisaoCidade(posicaoX, posicaoY, referencia, colisaoH, colisaoV); 
	}
	else if (strcmp(referencia, "inn") == 0 || strcmp(referencia, "castelo") == 0) {
		return colisaoCastelo(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "floresta1") == 0) {
		return colisaoFloresta1(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "floresta2") == 0) {
		return colisaoFloresta2(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "deserto") == 0) {
		return colisaoDeserto(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "deserto2") == 0) {
		return colisaoDeserto2(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "cenarioCaverna") == 0) {
		return colisaoCaverna1(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "cenarioCaverna2") == 0) {
		return colisaoCaverna2(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "cenarioCaverna3") == 0) {
		return colisaoCaverna3(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "Rocha") == 0) {
		return colisaoRocha(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "vulcao1") == 0) {
		return colisaoVulcao1(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if (strcmp(referencia, "vulcao2") == 0) {
		return colisaoVulcao2(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	else if ((strcmp(referencia, "batalhaMapa") == 0) || (strcmp(referencia, "batalhaDeserto") == 0) || (strcmp(referencia, "batalhaRocha") == 0) || (strcmp(referencia, "batalhaFloresta") == 0) || (strcmp(referencia, "batalhaCaverna") == 0)) {
		return colisaoBatalha(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
	}
	/*
	 * Tratamento de colisao entre o heroi e outros cenarios. 
	 * --> ATUALMENTE A GENTE NAO CAI NESTE CASO! MAS SE ALGUM CENARIO FOR INCLUSO É AQUI QUE SERA TRATADO.
	 * --> NA VERDADE SO DEIXEI AQUI PARA EVITAR MAIS UM WARNING...
	 */	
	else {
		PA_OutputText(0, 1, 1, "BATALHA!");
		
		destino = colisaoHeroi(posicaoX, posicaoY, referencia, colisaoH, colisaoV);
			
		if (destino == 0)
			return FALSO;
		else if (destino == 1)
			return VERDADEIRO;

	}
}

/* Verifica se dois personagens colidem */
u8 colidePersonagem(s32 posicao1X, s32 posicao1Y, s32 posicao2X, s32 posicao2Y, u8 raio)
{
	if(PA_Distance(posicao1X, posicao1Y, posicao2X, posicao2Y) < raio*raio)
		return VERDADEIRO;
		
	return FALSO;
}
