/*=========================================================================
This file is part of Broken Soul.

    Broken Soul is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	Broken Soul is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Broken Soul.  If not, see <http://www.gnu.org/licenses/>.
============================================================================*/

#ifndef HEROI_HPP
#define HEROI_HPP

#include "efs_lib.h"         // EFS
#include "nf_lib.h"              // NightFox's libs
#include "nf_easylib.h"

//#include "Inimigo.hpp"
#include "Persona.hpp"
#include "definicoes.hpp"
#include "colisao.hpp"
#include "Sound.hpp"
#include "Cenario.hpp"
#include "transicao.hpp"
#include "ListaLigada.hpp"
#include "Magia.hpp"

/* Heranca */
class Heroi : public Persona {
	public:
		s8  getMp();
		s8  getMpMax();
		s8  getHpMax();
		s8  getPtsDefesa();
		s8  getPtsMagia();
		s8  getPtsEvasao();
		s8  getTrocouCenario();
		s32 getDinheiro();		
		s32 getNivel();		
		s32 getXPMagia();
		s32 getXPMagiaMax();
		s32 getNivelMagia();
		s32 getAngulo();
		bool getVoltaX(u8 pos);
		bool getVoltaY(u8 pos);
		bool getReconhecendoCirc();
		u8 getXReco();
		u8 getYReco();

		void setReconhecendoCirc(bool rec);
		void setVoltaX(u8 pos, bool x);
		void setVoltaY(u8 pos, bool y);
		void setXReco(u8 x);
		void setYReco(u8 y);
		void setAngulo(s32 novoAngulo);
		void setMp(s8 mp);		
		void setMpMax(s8 mp);		
		void setHpMax(s8 hp);
		void setPtsMagia (s8 magia);
		void setDinheiro(s32 dinheiro);
		void setNivel(s32 nivel);
		void setXPMagia(s32 xp_magia);
		void setXPMagiaMax(s32 xp_magia_max);
		void setPtsDefesa (s8 defesa);
		void setPtsEvasao (s8 evasao);
		void setNivelMagia(s32 nivel_magia);
		void setTrocouCenario(s8 trocouCenario);
		void setUsandoEspiritoDoFogo(bool usando);
		void vira(u16 angulo);
		void pahra();
		void atualizaPosicoes(u16 angulo);
		void movimenta(u16 angulo, const char* referencia);
		void movimentaParaLadoDireito(u16 angulo);
		void movimentaParaLadoEsquerdo(u16 angulo);
		void movimentaParaCima(u16 angulo);
		void movimentaParaBaixo(u16 angulo);
		void ataca();
		void lancaMagia(Magia *magia);
		void carregaSprite(u8 tipoSprite, u8 descarrega);
		void inicializa(s32 screenPosX, s32 screenPosY);
		void posicaoCenario(s16 pos_cenario[]);
		void duploClique();
		

		/*funcoes que controlam pontos variaveis*/
		void aumentaHp(u16 mais_hp);
		void diminuiHp(u16 menos_hp);
		void aumentaMp(u8 mais_mp);
		void diminuiMp(u8 menos_mp);
		void aumentaExperiencia(s32 mais_exp);
		void zeraExperiencia();
		void aumentaExperienciaMagica(s32 mais_exp_mag);
		void zeraExperienciaMagica();

		/*funcoes que controlam ganho de pontos permanente*/
		void aumentaNivel(s32 nivelAdicional);
		void aumentaMp(s8 mpAdicional);
		void aumentaPtsDefesa(s8 defesaAdicional);
		void aumentaPtsMagia(s8 ptsMagiaAdicional);
		void aumentaPtsEvasao(s8 evasaoAdicional);
		void aumentaDinheiro(s32 dinheiroAdicional);

		void anda(s16 pos_cenario[], Cenario *cenario);
		void setDirecaoColisaoHorizontal(u8 direcao);
		void setDirecaoColisaoVertical(u8 direcao);

		s32 getPosicaoTelaAnteriorX();
		void setPosicaoTelaAnteriorX(s32 posicaoX);

		s32 getPosicaoTelaAnteriorY();
		void setPosicaoTelaAnteriorY(s32 posicaoY);

		s32 getPosicaoCenarioAnteriorX();
		void setPosicaoCenarioAnteriorX(s32 posicaoX);

		s32 getPosicaoCenarioAnteriorY();
		void setPosicaoCenarioAnteriorY(s32 posicaoY);

		s32 getScrollAnteriorY();
		void setScrollAnteriorY(s32 posicaoY);
		s32 getScrollAnteriorX();
		void setScrollAnteriorX(s32 posicaoX);

		u16 getContTomouDano();
		void setContTomouDano(u16 cont);

		bool getTomouDano();
		bool getUsandoEspiritoDoFogo();
		void setTomouDano(bool tomou);
		ListaLigada *getMochila();

		s32 getExperienciaAcumulada();
		void setExperienciaAcumulada(s32 experiencia);

		void setDuploClique (u8 add);
		u8 getDuploClique ();

		void espadada();
		void golpeGiratorio();

		void intencaoGolpeGiratorio();
		void iniciaVarRecoCirc();
		void iniciaRecoCirc();
		void iniciaVetRecCirc();
		bool reconhecendoCirc();

	private:
		s32 _dinheiro,
			_nivel,
			_xp_magia,
			_xp_magia_max,
			_nivel_magia,
			angulo,
			_posicaoCenarioAnterior[2],
			_posicaoTelaAnterior[2],
			_scrollAnterior[2],
			_experienciaAcumulada;

		u8  _duplo_clique,
			_x_reco,
			_y_reco;

		bool _tomouDano,
			 _usando_esp_fogo,
			 _voltaX[3],
			 _voltaY[2],
			 _reconhecendoCirc;
		
		s8	_pts_defesa,
			_pts_magia,
			_pts_evasao,
			_mp_max,
			_mp,
			_hp_max,
			_trocou_cenario;

		u16 _contTomouDano;

		ListaLigada *mochila;
};

#endif
