#ifndef __NF_EASYFAT_H__
#define __NF_EASYFAT_H__

#ifdef __cplusplus
extern "C" {
#endif





	// NightFox LIB - Easy FAT Functions
	// Requires PA_LIB, NODA AS_LIB & EFS 2.0.
	// Code by NightFox
	// http://blogs.gamefilia.com/knightfox
	// Version BETA.3.c.081117





	// Include required libs
	#include <PA9.h>			// PA_LIB


	// Define the sprite gfx control structures
	typedef struct {	// Sprite's Gfx Info
		char name[32];	// Sprite Gfx name (32 chars max)
		u8 slot;		// Slot state (free or in use)
		u8 size_x;		// Size X of the Gfx
		u8 size_y;		// Size Y of the Gfx
		u8 use[2];		// Times this GFX is used on each screen
	} NF_TYPE_SPRITE_GFX_INFO;
	extern NF_TYPE_SPRITE_GFX_INFO NF_SPRITE_GFX_INFO[NF_SPRITE_SLOTS];

	// Define the sprite palette control structures
	typedef struct {	// Sprite's Palette Info
		char name[32];	// Sprite Palette name (32 chars max)
		u8 slot;		// Slot state (free or in use)
		u8 use[2];		// Times this palette is used in each screen
	} NF_TYPE_SPRITE_PALETTE_INFO;
	extern NF_TYPE_SPRITE_PALETTE_INFO NF_SPRITE_PALETTE_INFO[NF_PALETTE_SLOTS];

	// Define the sprite control structures
	typedef struct {	// Sprite's general Info
		char name[32];	// Sprite name (32 chars max)
		u8 slot;		// Slot state (free or in use)
		u8 size_x;		// Size X of the Sprite
		u8 size_y;		// Size Y of the Sprite
		u16 gfx;		// GFX used
		u8 gfxslot;		// Slot number of GFX used
		u8 pal;			// Palette for this Sprite
		u8 palslot;		// Slot number of PAL used
	} NF_TYPE_SPRITE_INFO;
	extern NF_TYPE_SPRITE_INFO NF_SPRITE_INFO[128][2];

	// Control de las paletas en uso
	extern u8 NF_EASY_PALETTE[16][2];





	// Function NF_EasyClearAll();
	void NF_EasyClearAll(void);
	/*
	Cleans all RAM and VRAM of previous loaded files.
	Use this to ensure you clean up all buffers and structures.
	*/





	// Function NF_EasyInit();
	void NF_EasyInit(void);
	/*
	Initialitzes NightFox Easy Lib.
	Use this once before use any function of Easy lib.
	*/





	// Function NF_EasyLoadGfx();
	void NF_EasyLoadGfx(const char* referencia, const char* nombre, u8 size_x, u8 size_y);
	/*
	Load a GFX from EFS or FAT to RAM and asign a name to it for use later to create a Sprite.
	NF_EasyLoadGfx(
				"name you like here",
				"filename of GFX with path"
				size X in pixels,
				size Y in pixels
				);
	Put all Gfx on SPRITES folder.
	NF_EasyLoadGfx("ball", "stage1/blueball", 32, 32);
	*/




	// Function NF_EasyLoadPal();
	void NF_EasyLoadPal(const char* referencia, const char* nombre);
	/*
	Load a Palette from EFS or FAT to RAM and asign a name to it for use later to create a Sprite.
	NF_EasyLoadPal(
				"name you like here",
				"filename of palette with path");
	Put all Palettes on SPRITES folder.
	NF_EasyLoadPal("player", "stage1/playerblue");
	*/




	// Function NF_EasyCreateSprite();
	void NF_EasyCreateSprite(u8 pantalla, const char* referencia, const char* grafico, const char* paleta, s16 pos_x, s16 pos_y);
	/*
	Creates a sprite on given screen, using graphic and palette specified.
	Load grafic first with NF_EasyLoadGfx(); and palette with NF_EasyLoadPal();
	NF_EasyCreateSprite(
						u8 screen,
						"name for sprite",
						"gfx of sprite",
						"palete of sprite",
						s16 position x,
						s16 position y);
	NF_EasyCreateSprite(1, "mysprite", "ball", "player", 100, 70);
	*/





	// Function NF_EasyDeleteSprite();
	void NF_EasyDeleteSprite(u8 pantalla, const char* referencia);
	/*
	Deletes the sprite with the specified name
	NF_EasyDeleteSprite(
						u8 screen,
						"name of the sprite");
	NF_EasyDeleteSprite(1, "mysprite");
	*/





	// Function NF_EasyDeleteGfx();
	void NF_EasyDeleteGfx(const char* referencia);
	/*
	Deletes the GFX loaded with NF_EasyLoadGfx(); and frees the RAM in use.
	You can do this after you has ben created all Sprites that use this
	GFX to save some RAM for other things.
	void NF_EasyDeleteGfx("name of the GFX");
	NF_EasyDeleteGfx("ball");
	*/





	// Function NF_EasyDeletePal();
	void NF_EasyDeletePal(const char* referencia);
	/*
	Deletes de Palette loaded with NF_EasyLoadPal(); and frees the RAM in use.
	You can do this after you has ben created all Sprites that use this
	Palette to save some RAM for other things.
	void NF_EasyDeletePal("name of the palette");
	NF_EasyDeletePal("player");
	*/





#ifdef __cplusplus
}
#endif


#endif
