#ifndef __NF_FAT_H__
#define __NF_FAT_H__

#ifdef __cplusplus
extern "C" {
#endif





	// NightFox LIB - FAT Functions
	// Requires PA_LIB, NODA AS_LIB & EFS 2.0.
	// Code by NightFox
	// http://blogs.gamefilia.com/knightfox
	// Version BETA.3.c.081117





	// Include required libs
	#include <PA9.h>			// PA_LIB

	// Define SLOT numbers
	#define NF_BG_SLOTS 32				// Slots for BG's
	#define NF_SFX_SLOTS 32				// Slots for sound effects
	#define NF_CMAP_SLOTS 16			// Slots for colision maps
	#define NF_SPRITE_SLOTS 254			// Slots for sprites
	#define NF_PALETTE_SLOTS 32			// Slots for palettes (sprites)


	// Memory Buffers (Bg's) [32] - NF_BG_SLOTS
	extern char* NF_BUFFER_INFO[NF_BG_SLOTS];		
	extern char* NF_BUFFER_MAP[NF_BG_SLOTS];		
	extern char* NF_BUFFER_PAL[NF_BG_SLOTS];
	extern char* NF_BUFFER_TILES[NF_BG_SLOTS];

	typedef struct {	// Bg's Info
		char name[32];	// Bg name (32 chars max)
		u32 tiles;		// Tiles
		u32 map;		// Map
		u8 slot;		// Slot state (free or in use)
	} NF_TYPE_BGINFO;
	extern NF_TYPE_BGINFO NF_BGINFO[NF_BG_SLOTS];


	// Memory Buffers (RAW sound effects) [32] - NF_SFX_SLOTS
	extern char* NF_BUFFER_SFX[NF_SFX_SLOTS];

	typedef struct {	// Sound Info
		char name[32];	// Sound name (32 chars max)
		u32 size;		// Sound size
		u8 slot;		// Slot state (free or in use)
		u8 chan;		// Channel asigned to sound
	} NF_TYPE_SFXINFO;
	extern NF_TYPE_SFXINFO NF_SFXINFO[NF_SFX_SLOTS];


	// Memory Buffers (Colision maps)
	extern char* NF_CMAP[NF_CMAP_SLOTS];

	typedef struct {	// Colision maps info
		char name[32];	// Name of colision map (32 chars max)
		u32 size;		// Wide of map in tiles (pixels / 8)
		u8 slot;		// Slot state (free or in use)
	} NF_TYPE_CMAPINFO;
	extern NF_TYPE_CMAPINFO NF_CMAPINFO[NF_CMAP_SLOTS];


	// Memory buffers (Sprites) [64] - NF_SPRITE_SLOTS
	extern char* NF_SPRITE_GFX[NF_SPRITE_SLOTS];
	// Memory buffers (Palettes) [16] - NF_PALETTE_SLOTS
	extern char* NF_SPRITE_PALETTE[NF_PALETTE_SLOTS];


	// # Flashcard slot type
	extern u8 NF_SLOT_TYPE;


	// Root folder if using FAT
	extern char NF_ROOT_FOLDER[32];





	// Function NF_InitFAT();
	void NF_InitFAT(void);
	/*
	Try to initialitze EFS & FAT.
	EFS in default.
	Gives a error and stops running if init was failed.
	Sets the NF_SLOT_TYPE variable to slot type detected (1 or 2).
	*/





	// Function NF_SetFatRoot();
	void NF_SetFatRoot(const char* nombre);
	/*
	Sets the ROOT path folder of your app or game.
	Set it to "EFS" if you're using EFS.
	NF_SetFatRoot("mygame");
	Sets the "mygame" folder as root.
	NF_SetFatRoot("EFS");
	Sets the root folder to NULL to use EFS.
	*/





	// Function NF_InitBuffers();
	void NF_InitBuffers(void);
	/*
	Init all buffers.
	Do this once, before use any buffer.
	*/



	// Function NF_FreeBuffers();
	void NF_FreeBuffers(void);
	/*
	Free all buffers.
	Cleans all memory used also.
	*/



	// Function NF_LoadFatBg();
	void NF_LoadFatBg(const char* referencia, const char* nombre);
	/*
	Load a TILED Bg from EFS or FAT to RAM and asign a name to it.
	NF_LoadFatBg(
				"name you like here",
				"filename of map with path");
	Put all bg's on GFX folder.
	NF_LoadFatBg("sky", "stage1/skybg001");
	*/



	// Function NF_LoadRamBg();
	void NF_LoadRamBg(u16 pantalla, u16 fondo, const char* referencia);
	/*
	Load a Bg from RAM to VRAM. Load it before with NF_LoadFatBg();
	NF_LoadRamBg(
				screen,
				bg num,
				"name you gived to map");
	NF_LoadRamBg(1, 3, "sky");
	*/



	// Function NF_UnloadBg();
	void NF_UnloadBg(const char* referencia);
	/*
	Unload from RAM a Bg loaded with NF_LoadFatBg();
	Frees the RAM used and marks the slot as free.
	NF_UnloadBg("name you gived to map");
	NF_UnloadBg("sky");
	*/



	// Function NF_LoadFatSfx();
	void NF_LoadFatSfx(const char* referencia, const char* nombre);
	/*
	Load a RAW file from EFS or FAT to RAM, and asign a name to it.
	NF_LoadFatSfx(
				"name you like here",
				"filename of RAW with path");
	Put all sounds on SFX folder.
	NF_LoadFatSfx("thunder", "stage2/soundfile003");
	*/



	// Function NF_PlaySfx();
	void NF_PlaySfx(const char* referencia);
	/*
	Play a RAW file loaded in RAM with NF_LoadFatSfx();
	NF_PlaySfx("thunder");
	*/


	// Function NF_StopSfx();
	void NF_StopSfx(const char* referencia);
	/*
	Stops the sound played with NF_PlaySfx();
	NF_StopSfx("thunder");
	*/



	// Function NF_UnloadSfx();
	void NF_UnloadSfx(const char* referencia);
	/*
	Unload and frees RAM of RAW file loaded before with NF_LoadFatSfx();
	NF_UnloadSfx("thunder");
	*/



	// Function NF_LoadCollisionMap();
	void NF_LoadCollisionMap(const char* referencia, const char* nombre, u16 mapsize);
	/*
	Load a collision map from EFS or FAT.
	NF_LoadCollisionMap(
						"name you want for map",
						"filename with path",
						u16 map size in tiles (pixels wide / 8) );
	Put all your collision maps in MAPS folder.
	NF_LoadCollisionMap("dungeon1", "stage5/dunmap01", 32);
	*/



	// Function NF_CollisionMapSlot();
	extern u8 NF_CollisionMapSlot(const char* referencia);
	/*
	Returns the slot used by map loaded with NF_LoadCollisionMap(); to use it later with NF_GetTile();
	u8 myslot = NF_CollisionMapSlot("dungeon1");
	*/



	// Function NF_GetTile();
	extern u16 NF_GetTile(u8 slot, u16 x, u16 y);
	/*
	Returns the tile of colision map on slot selected.
	Use NF_CollisionMapSlot(); to get the slot number of a map.
	u16 tile = NF_GetTile(myslot, 100, 130);
	*/



	// Function NF_UnloadCollisionMap();
	void NF_UnloadCollisionMap(const char* referencia);
	/*
	Unload and frees RAM used of a collision map.
	NF_UnloadCollisionMap("dungeon1");
	*/


	// Function NF_LoadFatSprite();
	void NF_LoadFatSprite(u8 slot, const char* nombre);
	/*
	Load a Sprite from FAT or EFS to RAM into selected slot
	NF_LoadFatSprite(
					u8 slot number,
					"filename of sprite with path");
	Put all the sprites into SPRITES folder.
	NF_LoadFatSprite(32, "stage4/spaceship");
	This stores "spaceship" sprite into slot n32.
	Use the (void*)NF_SPRITE_GFX[32] variable to create it.
	*/



	// Funtion NF_LoadFatPalette();
	void NF_LoadFatPalette(u8 slot, const char* nombre); 
	/*
	Load a Palette from FAT or EFS to RAM into selected slot
	NF_LoadFatPalette(
					u8 slot number,
					"filename with path");
	Put all palettes into SPRITES folder.
	NF_LoadFatPalette(3, "stage4/player");
	This stores "player" palette into slot n3.
	Use the (void*)NF_SPRITE_PALETTE[3] variable to load it.
	*/



	// Function NF_FreeFatSprite();
	void NF_FreeFatSprite(u8 slot);
	/*
	Unload a loaded sprite from RAM with NF_LoadFatSprite().
	Do this after transfer it to VRAM.
	NF_FreeFatSprite(u8 slot number);
	NF_FreeFatSprite(32);
	*/



	// Function NF_FreeFatPalette();
	void NF_FreeFatPalette(u8 slot);
	/*
	Unload a loaded palette from RAM with NF_LoadFatPalette().
	Do this after transfer it to VRAM.
	NF_FreeFatPalette(u8 slot number);
	NF_FreeFatPalette(3);
	*/




#ifdef __cplusplus
}
#endif


#endif

