#ifdef __cplusplus
extern "C" {
#endif





	// NightFox LIB - Easy FAT Functions
	// Requires PA_LIB, NODA AS_LIB & EFS 2.0.
	// Code by NightFox
	// http://blogs.gamefilia.com/knightfox
	// Version BETA.3.c.081117





	// Incluye las librerias
	#include <PA9.h>			// Librerias PA_LIB
	#include <fat.h>			// Librerias adicionales del DevKit
	#include <unistd.h>			// Librerias UNISTD
	#include "include/efs_lib.h"		// Librerias EFS

	#include "include/nf_common.h"		// Librerias comunes de NightFox
	#include "include/nf_fat.h"			// Librerias FAT de NightFox
	#include "include/nf_easyfat.h"		// Librerias EasyFAT de NightFox
	#include "include/nf_easycommon.h"	// Librerias EasyCommon de NightFox





	// Function NF_EasyMoveSprite();	// Mueve el Sprite a las coordenadas dadas
	void NF_EasyMoveSprite(u8 pantalla, const char* referencia, s16 x, s16 y) {

		// Variables
		u8 slot;

		// Busca el Sprite
		slot = NF_EasyGetSpriteId(pantalla, referencia);

		// Posicionalo en pantalla
		PA_SetSpriteXY(pantalla, slot, x, y);

	}





	// Function NF_EasySetSpriteFrame();
	void NF_EasySetSpriteFrame(u8 pantalla, const char* referencia, u16 frame) {
		
		// Variables
		u8 sprite;
		u8 size_x;
		u8 size_y;

		// Obten la ID del sprite
		sprite = NF_EasyGetSpriteId(pantalla, referencia);

		// Obten las dimensiones del sprite
		size_x = NF_SPRITE_INFO[sprite][pantalla].size_x;
		size_y = NF_SPRITE_INFO[sprite][pantalla].size_y;

		// Pon el Sprite en el frame correspondiente
		PA_SetSpriteAnimEx(pantalla, sprite, size_x, size_y, 1, frame);

	}





	// Function NF_EasySetSpriteLayer();
	void NF_EasySetSpriteLayer(u8 pantalla, const char* referencia, u8 layer) {

		// Variables
		u8 sprite;

		// Obten la ID y el FLIP actual del sprite
		sprite = NF_EasyGetSpriteId(pantalla, referencia);

		// Pon la prioridad del sprite sobre el fondo dado
		PA_SetSpritePrio(pantalla, sprite, layer);

	}





	// Function NF_EasySpriteFlipH();
	void NF_EasySpriteFlipH(u8 pantalla, const char* referencia) {

		// Variables
		u8 sprite;
		u8 state;

		// Obten la ID y el FLIP actual del sprite
		sprite = NF_EasyGetSpriteId(pantalla, referencia);
		state = PA_GetSpriteHflip(pantalla, sprite);

		// Dale el flip H
		if (state == 0) {
			PA_SetSpriteHflip(pantalla, sprite, 1);
		} else {
			PA_SetSpriteHflip(pantalla, sprite, 0);
		}

	}





	// Function NF_EasySpriteFlipV();
	void NF_EasySpriteFlipV(u8 pantalla, const char* referencia) {

		// Variables
		u8 sprite;
		u8 state;

		// Obten la ID y el FLIP actual del sprite
		sprite = NF_EasyGetSpriteId(pantalla, referencia);
		state = PA_GetSpriteVflip(pantalla, sprite);

		// Dale el flip V
		if (state == 0) {
			PA_SetSpriteVflip(pantalla, sprite, 1);
		} else {
			PA_SetSpriteVflip(pantalla, sprite, 0);
		}

	}
	




	// Function NF_EasyGetSpriteId();	// Recupera la ID interna asignada al sprite solicitado
	u8 NF_EasyGetSpriteId(u8 pantalla, const char* referencia) {

		// Variables
		u8 n;
		u8 slot = 255;
		char nombre[32];

		// Busca el Sprite
		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar
		for (n = 0; n < 128; n ++) {
			if (strcmp(nombre, NF_SPRITE_INFO[n][pantalla].name) == 0) {	// Si coincide
				slot = n;	// Guarda el SLOT donde esta
				n = 128;	// Y sal
			}
		}
		if (slot == 255) {		// Si el sprite no se encuentra, da error
			NF_Error(4, referencia); 
		}

		return slot;		// Devuelve la ID asignada al sprite solicitado

	}





	// Function NF_EasyOverlapSprite();		// Comprueba si dos sprites se estan tocando
	bool NF_EasyOverlapSprite(u8 pantalla, const char* sprite_a, const char* sprite_b) {

		// Variables
		u8 slot_a;
		s16 xa;
		s16 ya;
		u8 size_xa;
		u8 size_ya;

		u8 slot_b;
		s16 xb;
		s16 yb;
		s8 size_xb;
		s8 size_yb;

		// Busca el Sprite
		slot_a = NF_EasyGetSpriteId(pantalla, sprite_a);
		slot_b = NF_EasyGetSpriteId(pantalla, sprite_b);

		// Obten las coordenadas
		xa = PA_GetSpriteX(pantalla, slot_a);
		ya = PA_GetSpriteY(pantalla, slot_a);
		xb = PA_GetSpriteX(pantalla, slot_b);
		yb = PA_GetSpriteY(pantalla, slot_b);

		// Obten las medidas
		size_xa = NF_SPRITE_INFO[slot_a][pantalla].size_x;
		size_ya = NF_SPRITE_INFO[slot_a][pantalla].size_y;
		size_xb = NF_SPRITE_INFO[slot_b][pantalla].size_x;
		size_yb = NF_SPRITE_INFO[slot_b][pantalla].size_y;

		// Se estan tocando ?
		if (
			(	// Borde Superior Izquierdo A
			((xa >= xb) && (xa <= (xb + size_xb)))
			&&
			((ya >= yb) && (ya <= (yb + size_yb)))
			)
			||	// Borde Superior Derecho A
			(
			(((xa + size_xa) > xb) && ((xa + size_xa) < (xb + size_xb)))
			&&
			((ya > yb) && (ya < (yb + size_yb)))
			)
			||
			(	// Borde Inferior Izquierdo A
			((xa > xb) && (xa < (xb + size_xb)))
			&&
			(((ya + size_ya) > yb) && ((ya + size_ya) < (yb + size_yb)))
			)
			||
			(	// Borde Inferior Derecho A
			(((xa + size_xa) > xb) && ((xa + size_xa) < (xb + size_xb)))
			&&
			(((ya + size_ya) > yb) && ((ya + size_ya) < (yb + size_yb)))
			)
			) {
			return true;
		} else {
			return false;
		}

	}





#ifdef __cplusplus
}
#endif
