#ifdef __cplusplus
extern "C" {
#endif





	// NightFox LIB - Easy FAT Functions
	// Requires PA_LIB, NODA AS_LIB & EFS 2.0.
	// Code by NightFox
	// http://blogs.gamefilia.com/knightfox
	// Version BETA.3.c.081117





	// Incluye las librerias
	#include <PA9.h>			// Librerias PA_LIB
	#include <fat.h>			// Librerias adicionales del DevKit
	#include <unistd.h>			// Librerias UNISTD
	#include "include/efs_lib.h"		// Librerias EFS

	#include "include/nf_common.h"		// Librerias comunes de NightFox
	#include "include/nf_fat.h"			// Librerias FAT de NightFox
	#include "include/nf_easyfat.h"		// Librerias EasyFAT de NightFox
	#include "include/nf_easycommon.h"	// Librerias EasyCommon de NightFox




	// Estructuras de control de los graficos de los sprites
	NF_TYPE_SPRITE_GFX_INFO NF_SPRITE_GFX_INFO[NF_SPRITE_SLOTS];
	// las paletas
	NF_TYPE_SPRITE_PALETTE_INFO NF_SPRITE_PALETTE_INFO[NF_PALETTE_SLOTS];
	// informacion del sprite en si
	NF_TYPE_SPRITE_INFO NF_SPRITE_INFO[128][2];
	// control de las paletas en uso
	u8 NF_EASY_PALETTE[16][2];





	// Function NF_EasyClearAll();
	void NF_EasyClearAll(void) {

		// Vacia los buffers y resetea las estructuras y variables
		NF_FreeBuffers();

		// Reinicializa las EasyFatLibs
		NF_EasyInit();

		// Reinicializa la VRAM
		PA_Init2D();					// Fuerza reinicializar el modo 2D
		PA_SetVideoMode(0, 0);			// Inicializa pantalla
		PA_SetVideoMode(1, 0);
		PA_ResetBgSysScreen(0);			// Reinicia el sistema de pantalla
		PA_ResetBgSysScreen(1);
		PA_ResetSpriteSysScreen(0);		// Reinicia el sistema de Sprites
		PA_ResetSpriteSysScreen(1);
		PA_DisableSpecialFx(0);			// Deshabilita los efectos especiales
		PA_DisableSpecialFx(1);

	}





	// Function NF_EasyInit();		// Inicializa todas las variables de la libreria
	void NF_EasyInit(void) {

		u8 n = 0;	// Variable de bucle
		u8 s = 0;

		// Variables de la funcion NF_EasyLoadGfx();
		for (n = 0; n < NF_SPRITE_SLOTS; n ++) {
			sprintf(NF_SPRITE_GFX_INFO[n].name, "xxxNULLxxx");
			NF_SPRITE_GFX_INFO[n].slot = 0;
			NF_SPRITE_GFX_INFO[n].size_x = 0;
			NF_SPRITE_GFX_INFO[n].size_y = 0;
			NF_SPRITE_GFX_INFO[n].use[0] = 0;		// Usos del grafico en cada pantalla
			NF_SPRITE_GFX_INFO[n].use[1] = 0;
		}

		// Variables de la funcion NF_EasyLoadPal();
		for (n = 0; n < NF_PALETTE_SLOTS; n ++) {
			sprintf(NF_SPRITE_PALETTE_INFO[n].name, "xxxNULLxxx");
			NF_SPRITE_PALETTE_INFO[n].slot = 0;
			NF_SPRITE_PALETTE_INFO[n].use[0] = 0;	// Usos de la paleta en cada pantalla
			NF_SPRITE_PALETTE_INFO[n].use[1] = 0;
		}

		// Variables de la funcion NF_EasyCreateSprite();
		for (n = 0; n < 128; n ++) {
			for (s = 0; s < 2; s ++) {
				sprintf(NF_SPRITE_INFO[n][s].name, "xxxNULLxxx");
				NF_SPRITE_INFO[n][s].slot = 0;
				NF_SPRITE_INFO[n][s].size_x = 0;
				NF_SPRITE_INFO[n][s].size_y = 0;
				NF_SPRITE_INFO[n][s].gfx = 0;		// Grafico usado
				NF_SPRITE_INFO[n][s].gfxslot = 0;	// Slot del grafico usado
				NF_SPRITE_INFO[n][s].pal = 0;		// Paleta
				NF_SPRITE_INFO[n][s].palslot = 0;	// Slot de la paleta usada
			}	
		}

		// Paletas en uso
		for (n = 0; n < 16; n++) {
			NF_EASY_PALETTE[n][0] = 0;
			NF_EASY_PALETTE[n][1] = 0;
		}

	}





	// Function NF_EasyLoadGfx();	// Carga un grafico para sprites desde la FAT
	void NF_EasyLoadGfx(const char* referencia, const char* nombre, u8 size_x, u8 size_y) {

		// Busca un slot libre
		u8 n = 0;
		u8 slot = 255;
		for (n = 0; n < NF_SPRITE_SLOTS; n ++) {
			if (NF_SPRITE_GFX_INFO[n].slot == 0) {	// Si esta libre, asignalo y sal
				slot = n;
				NF_SPRITE_GFX_INFO[n].slot = 1;
				n = NF_SPRITE_SLOTS;
			}
		}
		if (slot == 255) {		// Si no hay ningun slot libre, error
			NF_Error(8, NULL);
		}

		NF_LoadFatSprite(slot, nombre);		// Carga el grafico en el slot correspondiente

		sprintf(NF_SPRITE_GFX_INFO[slot].name, "%s", referencia);		// Guarda el nombre del grafico
		NF_SPRITE_GFX_INFO[slot].size_x = size_x;						// Y las medidas
		NF_SPRITE_GFX_INFO[slot].size_y = size_y;

	}





	// Function NF_EasyLoadPal();	// Carga paleta para sprites desde la FAT
	void NF_EasyLoadPal(const char* referencia, const char* nombre) {

		// Busca un slot libre
		u8 n = 0;
		u8 slot = 255;
		for (n = 0; n < NF_PALETTE_SLOTS; n ++) {
			if (NF_SPRITE_PALETTE_INFO[n].slot == 0) {	// Si esta libre, asignalo y sal
				slot = n;
				NF_SPRITE_PALETTE_INFO[n].slot = 1;
				n = NF_PALETTE_SLOTS;
			}
		}
		if (slot == 255) {		// Si no hay ningun slot libre, error
			NF_Error(9, NULL);
		}

		NF_LoadFatPalette(slot, nombre);		// Carga la paleta en el slot correspondiente

		sprintf(NF_SPRITE_PALETTE_INFO[slot].name, "%s", referencia);		// Guarda el nombre de la paleta

	}





	// Function NF_EasyCreateSprite();	// Crea un sprite con el grafico y paleta especificados
	void NF_EasyCreateSprite(u8 pantalla, const char* referencia, const char* grafico, const char* paleta, s16 pos_x, s16 pos_y) {

		// Variables
		u8 n;
		char nombre[32];

		// Guarda slots de referencia
		u8 slot = 255;
		u8 slot_referencia = 255;
		u8 slot_grafico = 255;
		u8 slot_paleta = 255;
		u8 shape = 0;
		u8 size = 0;

		// Busca un slot libre
		for (n = 0; n < 128; n ++) {
			if (NF_SPRITE_INFO[n][pantalla].slot == 0) {	// Si esta libre, asignalo y sal
				NF_SPRITE_INFO[n][pantalla].slot = 1;
				slot_referencia = n;	// Guarda el slot que has usado
				n = 128;
			}
		}
		if (slot_referencia == 255) {		// Si no hay ningun slot libre, error
			NF_Error(10, NULL);
		}

		// Busca el GFX para el Sprite
		sprintf(nombre, "%s", grafico);		// Obten el nombre a buscar
		for (n = 0; n < NF_SPRITE_SLOTS; n ++) {
			if (strcmp(nombre, NF_SPRITE_GFX_INFO[n].name) == 0) {	// Si coincide
				slot_grafico = n;		// Guarda el SLOT donde esta
				n = NF_SPRITE_SLOTS;	// Y sal
			}
		}
		if (slot_grafico == 255) {		// Si el grafico no se encuentra, da error
			NF_Error(4, grafico); 
		}

		// Busca la Paleta para el Sprite
		sprintf(nombre, "%s", paleta);		// Obten el nombre a buscar
		for (n = 0; n < NF_PALETTE_SLOTS; n ++) {
			if (strcmp(nombre, NF_SPRITE_PALETTE_INFO[n].name) == 0) {	// Si coincide
				slot_paleta = n;		// Guarda el SLOT donde esta
				n = NF_PALETTE_SLOTS;	// Y sal
			}
		}
		if (slot_paleta == 255) {		// Si la paleta no se encuentra, da error
			NF_Error(4, paleta); 
		}

		// Calcula el Shape segun las medias
		if (NF_SPRITE_GFX_INFO[slot_grafico].size_x == NF_SPRITE_GFX_INFO[slot_grafico].size_y) {		// X == Y
			shape = 0;
		}
		if (NF_SPRITE_GFX_INFO[slot_grafico].size_x > NF_SPRITE_GFX_INFO[slot_grafico].size_y) {		// X > Y
			shape = 1;
		}
		if (NF_SPRITE_GFX_INFO[slot_grafico].size_x < NF_SPRITE_GFX_INFO[slot_grafico].size_y) {		// X < Y
			shape = 2;
		}

		// Calcula la medida segun el shape y la medida
		if (shape == 0) {		// X == Y
			if (NF_SPRITE_GFX_INFO[slot_grafico].size_x == 8) {
				size = 0;
			}
			if (NF_SPRITE_GFX_INFO[slot_grafico].size_x == 16) {
				size = 1;
			}
			if (NF_SPRITE_GFX_INFO[slot_grafico].size_x == 32) {
				size = 2;
			}
			if (NF_SPRITE_GFX_INFO[slot_grafico].size_x == 64) {
				size = 3;
			}
		}
		if (shape == 1) {		// X > Y
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 16) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 8)) {
				size = 0;
			}
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 32) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 8)) {
				size = 1;
			}
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 32) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 16)) {
				size = 2;
			}
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 64) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 32)) {
				size = 3;
			}
		}
		if (shape == 2) {		// X < Y
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 8) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 16)) {
				size = 0;
			}
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 8) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 32)) {
				size = 1;
			}
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 16) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 32)) {
				size = 2;
			}
			if ((NF_SPRITE_GFX_INFO[slot_grafico].size_x == 32) && (NF_SPRITE_GFX_INFO[slot_grafico].size_y == 64)) {
				size = 3;
			}
		}

		// Si no esta creado, crea el GFX en la VRAM
		if (NF_SPRITE_GFX_INFO[slot_grafico].use[pantalla] == 0) {
			// Crea el grafico
			NF_SPRITE_INFO[slot_referencia][pantalla].gfx = PA_CreateGfx(pantalla, (void*)NF_SPRITE_GFX[slot_grafico], shape, size, 1);
			// Y deja constancia
			NF_SPRITE_INFO[slot_referencia][pantalla].gfxslot = slot_grafico;
			NF_SPRITE_GFX_INFO[slot_grafico].use[pantalla] += 1;

		} else {
			// Busca que otro Sprite tiene el grafico
			slot = 255;
			for (n = 0; n < 128; n ++) {
				// Si encuentras el Sprite que lo contiene
				if (NF_SPRITE_INFO[n][pantalla].gfxslot == slot_grafico) {
					slot = n;	// Guardalo y sal
					n = 128;
				}
			}
			// Si no has encontrado el GFX, error grave de codigo
			if (slot == 255) {
				NF_Error(255, "NF_EasyCreateSprite();");
			}
			// Copia el GFX del Sprite existente
			NF_SPRITE_INFO[slot_referencia][pantalla].gfx = NF_SPRITE_INFO[slot][pantalla].gfx;
			// Y registralo
			NF_SPRITE_INFO[slot_referencia][pantalla].gfxslot = slot_grafico;
			NF_SPRITE_GFX_INFO[slot_grafico].use[pantalla] += 1;
		}


		// Si la paleta no esta cargada en memoria, cargala
		if (NF_SPRITE_PALETTE_INFO[slot_paleta].use[pantalla] == 0) {

			// Busca un slot libre para la paleta
			slot = 255;
			for (n = 0; n < 16; n ++) {
				if (NF_EASY_PALETTE[n][pantalla] == 0) {
					NF_EASY_PALETTE[n][pantalla] = 1;	// Marcala en uso
					slot = n;	// Guarda el slot
					n = 16;		// Y sal
				}
			}
			if (slot == 255) {		// Si no hay ningun slot libre, error
				NF_Error(11, NULL);
			}
			// Carga la paleta
			PA_LoadSpritePal(pantalla, slot, (void*)NF_SPRITE_PALETTE[slot_paleta]);
			// y registralo
			NF_SPRITE_INFO[slot_referencia][pantalla].pal = slot;
			NF_SPRITE_INFO[slot_referencia][pantalla].palslot = slot_paleta;
			NF_SPRITE_PALETTE_INFO[slot_paleta].use[pantalla] += 1;

		} else {	// Si ya esta en memoria, reutilizala

			// Busca que otro Sprite tiene la misma paleta
			slot = 255;
			for (n = 0; n < 128; n ++) {
				// Si encuentras el Sprite que la usa
				if (NF_SPRITE_INFO[n][pantalla].palslot == slot_paleta) {
					slot = n;	// Guardalo y sal
					n = 128;
				}
			}
			// Si no has encontrado la paleta, error grave de codigo
			if (slot == 255) {
				NF_Error(255, "NF_EasyCreateSprite();");
			}
			// Copia la paleta del Sprite existente
			NF_SPRITE_INFO[slot_referencia][pantalla].pal = NF_SPRITE_INFO[slot][pantalla].pal;
			// y registralo
			NF_SPRITE_INFO[slot_referencia][pantalla].palslot = slot_paleta;
			NF_SPRITE_PALETTE_INFO[slot_paleta].use[pantalla] += 1;

		}


		// Guarda los datos del Sprite
		sprintf(NF_SPRITE_INFO[slot_referencia][pantalla].name, "%s", referencia);
		NF_SPRITE_INFO[slot_referencia][pantalla].size_x = NF_SPRITE_GFX_INFO[slot_grafico].size_x;
		NF_SPRITE_INFO[slot_referencia][pantalla].size_y = NF_SPRITE_GFX_INFO[slot_grafico].size_y;

		// Y crealo
		PA_CreateSpriteFromGfx(
							pantalla,
							slot_referencia,
							NF_SPRITE_INFO[slot_referencia][pantalla].gfx,
							shape,
							size,
							1,
							NF_SPRITE_INFO[slot_referencia][pantalla].pal,
							pos_x,
							pos_y
							);

	}





	// Function NF_EasyDeleteSprite();	// Borra el Sprite
	void NF_EasyDeleteSprite(u8 pantalla, const char* referencia) {

		// Variables
		u8 slot;

		// Busca el Sprite
		slot = NF_EasyGetSpriteId(pantalla, referencia);

		// Informa al GFX que hay un uso menos
		NF_SPRITE_GFX_INFO[NF_SPRITE_INFO[slot][pantalla].gfxslot].use[pantalla] -= 1;
		// Si ya no hay ningun sprite que use el grafico
		if (NF_SPRITE_GFX_INFO[NF_SPRITE_INFO[slot][pantalla].gfxslot].use[pantalla] == 0) {
			// Borra el GFX
			PA_DeleteGfx(pantalla, NF_SPRITE_INFO[slot][pantalla].gfx);
		}

		// Informa a la PALETA que hay un uso menos
		NF_SPRITE_PALETTE_INFO[NF_SPRITE_INFO[slot][pantalla].palslot].use[pantalla] -= 1;
		// Si ningus sprite usa esta paleta
		if (NF_SPRITE_PALETTE_INFO[NF_SPRITE_INFO[slot][pantalla].palslot].use[pantalla] == 0) {
			// Marca esta paleta como disponible
			NF_EASY_PALETTE[NF_SPRITE_INFO[slot][pantalla].pal][pantalla] = 0;
		}

		// Borra el sprite
		PA_DeleteSprite(pantalla, slot);

		// Resetea los datos del sprite
		sprintf(NF_SPRITE_INFO[slot][pantalla].name, "xxxNULLxxx");
		NF_SPRITE_INFO[slot][pantalla].slot = 0;
		NF_SPRITE_INFO[slot][pantalla].size_x = 0;
		NF_SPRITE_INFO[slot][pantalla].size_y = 0;
		NF_SPRITE_INFO[slot][pantalla].gfx = 0;		// Grafico usado
		NF_SPRITE_INFO[slot][pantalla].gfxslot = 0;	// Slot del grafico usado
		NF_SPRITE_INFO[slot][pantalla].pal = 0;		// Paleta
		NF_SPRITE_INFO[slot][pantalla].palslot = 0;	// Slot de la paleta usada

	}





	// Function NF_EasyDeleteGfx();	// Borra de la RAM el GFX solicitado
	void NF_EasyDeleteGfx(const char* referencia) {

		// Variables
		u8 n;
		u8 slot = 255;
		char nombre[32];

		// Busca el GFX
		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar
		for (n = 0; n < NF_SPRITE_SLOTS; n ++) {
			if (strcmp(nombre, NF_SPRITE_GFX_INFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_SPRITE_SLOTS;	// Y sal
			}
		}
		if (slot == 255) {		// Si el grafico no se encuentra, da error
			NF_Error(4, referencia); 
		}

		NF_FreeFatSprite(slot);		// Borra el GFX cargado en la RAM

		// Resetea los parametros
		sprintf(NF_SPRITE_GFX_INFO[slot].name, "xxxNULLxxx");
		NF_SPRITE_GFX_INFO[slot].slot = 0;
		NF_SPRITE_GFX_INFO[slot].size_x = 0;
		NF_SPRITE_GFX_INFO[slot].size_y = 0;
		NF_SPRITE_GFX_INFO[slot].use[0] = 0;		// Usos del grafico en cada pantalla
		NF_SPRITE_GFX_INFO[slot].use[1] = 0;

	}





	// Function NF_EasyDeletePal();	// Borra de la RAM la Paleta solicitada
	void NF_EasyDeletePal(const char* referencia) {

		// Variables
		u8 n;
		u8 slot = 255;
		char nombre[32];

		// Busca la Paleta
		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar
		for (n = 0; n < NF_PALETTE_SLOTS; n ++) {
			if (strcmp(nombre, NF_SPRITE_PALETTE_INFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_PALETTE_SLOTS;	// Y sal
			}
		}
		if (slot == 255) {		// Si la paleta no se encuentra, da error
			NF_Error(4, referencia); 
		}

		NF_FreeFatPalette(slot);		// Borra la paleta indicada de la RAM

		// Resetea los parametros
		sprintf(NF_SPRITE_PALETTE_INFO[slot].name, "xxxNULLxxx");
		NF_SPRITE_PALETTE_INFO[slot].slot = 0;
		NF_SPRITE_PALETTE_INFO[slot].use[0] = 0;	// Usos de la paleta en cada pantalla
		NF_SPRITE_PALETTE_INFO[slot].use[1] = 0;

	}





#ifdef __cplusplus
}
#endif
