#ifdef __cplusplus
extern "C" {
#endif





	// NightFox LIB - FAT Functions
	// Requires PA_LIB, NODA AS_LIB & EFS 2.0.
	// Code by NightFox
	// http://blogs.gamefilia.com/knightfox
	// Version BETA.3.c.081117





	// Incluye las librerias
	#include <PA9.h>			// Librerias PA_LIB
	#include <fat.h>			// Librerias adicionales del DevKit
	#include <unistd.h>			// Librerias UNISTD
	#include "include/efs_lib.h"		// Librerias EFS

	#include "include/nf_common.h"		// Librerias comunes de NightFox
	#include "include/nf_fat.h"			// Librerias FAT de NightFox

	#include "include/nf_font.h"		// Fuentes propias





	// Memory Buffers (Bg's) [32] - NF_BG_SLOTS
	char* NF_BUFFER_INFO[NF_BG_SLOTS];		
	char* NF_BUFFER_MAP[NF_BG_SLOTS];		
	char* NF_BUFFER_PAL[NF_BG_SLOTS];
	char* NF_BUFFER_TILES[NF_BG_SLOTS];
	NF_TYPE_BGINFO NF_BGINFO[NF_BG_SLOTS];


	// Memory Buffers (RAW sound effects) [32] - NF_SFX_SLOTS
	char* NF_BUFFER_SFX[NF_SFX_SLOTS];
	NF_TYPE_SFXINFO NF_SFXINFO[NF_SFX_SLOTS];


	// Memory Buffers (Colision maps)
	char* NF_CMAP[NF_CMAP_SLOTS];
	NF_TYPE_CMAPINFO NF_CMAPINFO[NF_CMAP_SLOTS];


	// Memory buffers (Sprites) [64] - NF_SPRITE_SLOTS
	char* NF_SPRITE_GFX[NF_SPRITE_SLOTS];
	// Memory buffers (Palettes) [16] - NF_PALETTE_SLOTS
	char* NF_SPRITE_PALETTE[NF_PALETTE_SLOTS];


	// # Flashcard slot type
	u8 NF_SLOT_TYPE;


	// Root folder if using FAT
	char NF_ROOT_FOLDER[32];





	// Funcion NF_InitFAT();			// Inicializa las FATlib
	void NF_InitFAT(void) {

		// Inicializa el texto
		PA_SetVideoMode(0, 0);		// Inicializa las pantallas con el modo de video correspondiente
		PA_SetVideoMode(1, 0);
		PA_InitCustomText(0, 0, fuente_system);		// Inicializa el texto en ambas pantallas
		PA_InitCustomText(1, 0, fuente_system);		// En la capa 0
		PA_SetTextCol(0, 31, 31, 31);
		PA_EasyBgLoad(0, 3, bg_system);		// Carga el fondo
		PA_EasyBgLoad(1, 3, bg_system);
		PA_SetBrightness(0, -16);			// Brillo bajado
		PA_SetBrightness(1, -16);

		// Inicia el sistema FAT
		PA_OutputText(1, 1, 1, "INICIANDO SISTEMA DE ARCHIVOS.");		// Mensage de error en Castellano
		PA_OutputText(1, 1, 2, "ESTO PUEDE TARDAR CERCA DE");
		PA_OutputText(1, 1, 3, "1 MINUTO LA PRIMERA VEZ.");
		PA_OutputText(0, 1, 1, "STARTING FILE SYSTEM.");			// Mensage de error en ingles
		PA_OutputText(0, 1, 2, "THIS WILL TAKE ARROUND");
		PA_OutputText(0, 1, 3, "1 MINUTE THE FIRST TIME.");
		NF_FadeIn();

		if(EFS_Init(EFS_AND_FAT | EFS_DEFAULT_DEVICE, NULL)) {
			// Si has iniciado correctamente, define EFS por defecto
			chdir("efs:/");
			// Y detecta si el linker es SLOT 1 o SLOT 2
			if (strcmp(efs_path, "GBA ROM") == 0) {		// Si es SLOT 2 o emulador
				NF_SLOT_TYPE = 2;
			} else {		// Si es SLOT 1
				NF_SLOT_TYPE = 1;
			}
		} else {
			PA_ClearTextBg(1);
			PA_ClearTextBg(0);
			PA_OutputText(1, 1, 1, "SI LEES ESTE MENSAGE, TU");		// Mensage de error en Castellano
			PA_OutputText(1, 1, 2, "FLASHCARD NO ES COMPATIBLE");
			PA_OutputText(1, 1, 3, "O NO HAS APLICADO EL PARCHE");
			PA_OutputText(1, 1, 4, "DLDI CORRESPONDIENTE.");
			PA_OutputText(1, 1, 7, "SISTEMA DETENIDO.");	
			PA_OutputText(0, 1, 1, "IF YOU'RE READING THIS,");		// Mensage de error en ingles
			PA_OutputText(0, 1, 2, "YOUR FLASHCARD IS NOT");
			PA_OutputText(0, 1, 3, "COMPATIBLE OR YOU HAVEN'T");
			PA_OutputText(0, 1, 4, "PATCHED WITH RIGHT DLDI.");
			PA_OutputText(0, 1, 7, "SYSTEM STOPPED.");
			while(1) {
				PA_WaitForVBL();
			}
		}

		NF_FadeOut();

	}





	// Funcion NF_SetFatRoot();			// Define el ROOT si se usa FAT. Poner en NULL si se usa EFS
	void NF_SetFatRoot(const char* nombre) {

		char texto[32];
		sprintf(texto, "%s", nombre);

		if (strcmp(texto, "EFS") == 0) {	// Si se usa EFS, ponlo a vacio

			sprintf(NF_ROOT_FOLDER, "%s", "");
			chdir("efs:/");		// Y cambia el ROOT a EFS:/

		} else {				// Si se usa FAT, asignale el nombre deseado

			sprintf(NF_ROOT_FOLDER, "/%s", nombre);
			chdir("fat:/");		// Y cambia el ROOT a FAT:/

		}

	}





	// Funcion NF_InitBuffers();		// Inicializa los punteros de carga de los buffers
	void NF_InitBuffers(void) {

		u8 x;

		// Buffers de graficos
		for (x = 0; x < NF_BG_SLOTS; x ++) {
			NF_BUFFER_INFO[x] = NULL;		
			NF_BUFFER_MAP[x] = NULL;			
			NF_BUFFER_PAL[x] = NULL;
			NF_BUFFER_TILES[x] = NULL;
			sprintf(NF_BGINFO[x].name, "xxxNULLxxx");
			NF_BGINFO[x].tiles = 0;
			NF_BGINFO[x].map = 0;
			NF_BGINFO[x].slot = 0;
		}

		// Buffers de sonido
		for (x = 0; x < NF_SFX_SLOTS; x++) {
			NF_BUFFER_SFX[x] = NULL;
			sprintf(NF_SFXINFO[x].name, "xxxNULLxxx");
			NF_SFXINFO[x].size = 0;
			NF_SFXINFO[x].slot = 0;
			NF_SFXINFO[x].chan = 0;
		}

		// Mapas de colision
		for (x = 0; x < NF_CMAP_SLOTS; x++) {
			NF_CMAP[x] = NULL;
			sprintf(NF_CMAPINFO[x].name, "xxxNULLxxx");
			NF_CMAPINFO[x].size = 0;
			NF_CMAPINFO[x].slot = 0;
		}

		// Sprites
		for (x = 0; x < NF_SPRITE_SLOTS; x++) {
			NF_SPRITE_GFX[x] = NULL;
		}

		// Paletas
		for (x = 0; x < NF_PALETTE_SLOTS; x++) {
			NF_SPRITE_PALETTE[x] = NULL;
		}

	}





	// Funcion NF_FreeBuffers();		// Vacia todos los buffers
	void NF_FreeBuffers(void) {

		u8 x;

		// Buffers de graficos
		for (x = 0; x < NF_BG_SLOTS; x ++) {
			free(NF_BUFFER_INFO[x]);
			NF_BUFFER_INFO[x] = NULL;		
			free(NF_BUFFER_MAP[x]);
			NF_BUFFER_MAP[x] = NULL;			
			free(NF_BUFFER_PAL[x]);
			NF_BUFFER_PAL[x] = NULL;
			free(NF_BUFFER_TILES[x]);
			NF_BUFFER_TILES[x] = NULL;
			sprintf(NF_BGINFO[x].name, "xxxNULLxxx");
			NF_BGINFO[x].tiles = 0;
			NF_BGINFO[x].map = 0;
			NF_BGINFO[x].slot = 0;
		}

		// Buffers de sonido
		for (x = 0; x < NF_SFX_SLOTS; x++) {
			free(NF_BUFFER_SFX[x]);	
			NF_BUFFER_SFX[x] = NULL;
			sprintf(NF_SFXINFO[x].name, "xxxNULLxxx");
			NF_SFXINFO[x].size = 0;
			NF_SFXINFO[x].slot = 0;	
			NF_SFXINFO[x].chan = 0;
		}

		// Mapa de colisiones
		for (x = 0; x < NF_CMAP_SLOTS; x++) {
			free(NF_CMAP[x]);
			NF_CMAP[x] = NULL;
			sprintf(NF_CMAPINFO[x].name, "xxxNULLxxx");
			NF_CMAPINFO[x].size = 0;
			NF_CMAPINFO[x].slot = 0;
		}

		// Sprites
		for (x = 0; x < NF_SPRITE_SLOTS; x++) {
			free(NF_SPRITE_GFX[x]);
			NF_SPRITE_GFX[x] = NULL;
		}

		// Paletas
		for (x = 0; x < NF_PALETTE_SLOTS; x++) {
			free(NF_SPRITE_PALETTE[x]);
			NF_SPRITE_PALETTE[x] = NULL;
		}

	}





	// Funcion NF_LoadFatBg();		// Carga un fondo desde la FAT
	// NF_LoadFatBg(slot, referencia, nombre_archivo)
	void NF_LoadFatBg(const char* referencia, const char* nombre) {

		// Busca un slot libre
		u8 n = 0;
		u8 slot = 255;
		for (n = 0; n < NF_BG_SLOTS; n ++) {
			if (NF_BGINFO[n].slot == 0) {	// Si esta libre, asignalo y sal
				slot = n;
				NF_BGINFO[n].slot = 1;
				n = NF_BG_SLOTS;
			}
		}
		if (slot == 255) {		// Si no hay ningun slot libre, error
			NF_Error(5, NULL);
		}

		free(NF_BUFFER_INFO[slot]);		// Vacia el buffer
		NF_BUFFER_INFO[slot] = NULL;		
		free(NF_BUFFER_MAP[slot]);
		NF_BUFFER_MAP[slot] = NULL;
		free(NF_BUFFER_PAL[slot]);
		NF_BUFFER_PAL[slot] = NULL;
		free(NF_BUFFER_TILES[slot]);
		NF_BUFFER_TILES[slot] = NULL;

		u32 medida_info = 0;		// Tamao del archivo a cargar
		u32 medida_map = 0;
		u32 medida_pal = 0;
		u32 medida_tiles = 0;

		FILE* file_info;		// Nombres de archivo
		FILE* file_map;
		FILE* file_pal;
		FILE* file_tiles;

		char filename[256];		// Nombre del archivo

		// Archivo INFO
		sprintf(filename, "%s/gfx/%s_Info.bin", NF_ROOT_FOLDER, nombre);	// Abre Info.bin
		file_info = fopen(filename, "rb");
		if (file_info) {		// Si el archivo "Info" existe...
			// Obten el tamao del archivo
			fseek(file_info, 0, SEEK_END);
			medida_info = ftell(file_info);
			rewind(file_info);
			// Reserva el espacio en RAM
			NF_BUFFER_INFO[slot] = (char*) calloc (medida_info, sizeof(char));
			if (NF_BUFFER_INFO[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_BUFFER_INFO[slot], 1, medida_info, file_info);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_info);		// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

		// Archivo MAP
		sprintf(filename, "%s/gfx/%s_Map.bin", NF_ROOT_FOLDER, nombre);	// Abre Map.bin
		file_map = fopen(filename, "rb");
		if (file_map) {		// Si el archivo "Map" existe...
			// Obten el tamao del archivo
			fseek(file_map, 0, SEEK_END);
			medida_map = ftell(file_map);
			rewind(file_map);
			// Reserva el espacio en RAM
			NF_BUFFER_MAP[slot] = (char*) calloc (medida_map, sizeof(char));
			if (NF_BUFFER_MAP[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_BUFFER_MAP[slot], 1, medida_map, file_map);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_map);		// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

		// Archivo PAL
		sprintf(filename, "%s/gfx/%s_Pal.bin", NF_ROOT_FOLDER, nombre);	// Abre Pal.bin
		file_pal = fopen(filename, "rb");
		if (file_pal) {		// Si el archivo "Pal" existe...
			// Obten el tamao del archivo
			fseek(file_pal, 0, SEEK_END);
			medida_pal = ftell(file_pal);
			rewind(file_pal);
			// Reserva el espacio en RAM
			NF_BUFFER_PAL[slot] = (char*) calloc (medida_pal, sizeof(char));
			if (NF_BUFFER_PAL[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_BUFFER_PAL[slot], 1, medida_pal, file_pal);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_pal);		// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

		// Archivo TILES
		sprintf(filename, "%s/gfx/%s_Tiles.bin",NF_ROOT_FOLDER, nombre);	// Abre Tiles.bin
		file_tiles = fopen(filename, "rb");
		if (file_tiles) {		// Si el archivo "Tiles" existe...
			// Obten el tamao del archivo
			fseek(file_tiles, 0, SEEK_END);
			medida_tiles = ftell(file_tiles);
			rewind(file_tiles);
			// Reserva el espacio en RAM
			NF_BUFFER_TILES[slot] = (char*) calloc (medida_tiles, sizeof(char));
			if (NF_BUFFER_TILES[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_BUFFER_TILES[slot], 1, medida_tiles, file_tiles);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_tiles);		// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

		NF_BGINFO[slot].tiles = medida_tiles;	// Guarda las medidas del mapa
		NF_BGINFO[slot].map = medida_map;
		sprintf(NF_BGINFO[slot].name, "%s", referencia);

	}





	// Funcion NF_LoadRamBg()
	void NF_LoadRamBg(u16 pantalla, u16 fondo, const char* referencia) {

		// Busca si existe el archivo
		u8 n;
		u8 slot = 255;
		char nombre[32];

		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar

		for (n = 0; n < NF_BG_SLOTS; n ++) {
			if (strcmp(nombre, NF_BGINFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_BG_SLOTS;	// Y sal
			}
		}

		if (slot == 255) {		// Si el fondo no se encuentra, da error
			NF_Error(4, referencia); 
		}

		PA_DeleteBg(pantalla, fondo);			// Borra el fondo

		// Transfiere el fondo a la VRAM
		PA_EasyBgLoadEx(
						pantalla,						// U8 Screen
						fondo,							// U8 BG Number
						(u32*)NF_BUFFER_INFO[slot],		// (u32*) Infos
						(void*)NF_BUFFER_TILES[slot],	// (void*) Tiles
						NF_BGINFO[slot].tiles,			// U32 Tile Size
						(void*)NF_BUFFER_MAP[slot],		// (void*) Map
						NF_BGINFO[slot].map,			// U32 Map Size (Puede que necesite "/ 2" o "<< 1")
						(void*)NF_BUFFER_PAL[slot]		// (void*) Palette
						);

	}





	// Funcion NF_UnloadBg()
	void NF_UnloadBg(const char* referencia) {

		// Busca si existe el fondo
		u8 n;
		u8 slot = 255;
		char nombre[32];

		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar

		for (n = 0; n < NF_BG_SLOTS; n ++) {
			if (strcmp(nombre, NF_BGINFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_BG_SLOTS;	// Y sal
			}
		}

		if (slot == 255) {		// Si el fondo no se encuentra, da error
			NF_Error(4, referencia); 
		}

		// Descarga el fondo de la RAM
		free(NF_BUFFER_INFO[slot]);
		NF_BUFFER_INFO[slot] = NULL;		
		free(NF_BUFFER_MAP[slot]);
		NF_BUFFER_MAP[slot] = NULL;			
		free(NF_BUFFER_PAL[slot]);
		NF_BUFFER_PAL[slot] = NULL;
		free(NF_BUFFER_TILES[slot]);
		NF_BUFFER_TILES[slot] = NULL;
		sprintf(NF_BGINFO[slot].name, "xxxNULLxxx");
		NF_BGINFO[slot].tiles = 0;
		NF_BGINFO[slot].map = 0;
		NF_BGINFO[slot].slot = 0;

	}





	// Funcion NF_LoadFatSfx();	// Carga un archivo RAW desde la FAT
	void NF_LoadFatSfx(const char* referencia, const char* nombre) {

		// Busca un slot libre
		u8 n = 0;
		u8 slot = 255;
		for (n = 0; n < NF_SFX_SLOTS; n ++) {
			if (NF_SFXINFO[n].slot == 0) {	// Si esta libre, asignalo y sal
				slot = n;
				NF_SFXINFO[n].slot = 1;
				n = NF_SFX_SLOTS;
			}
		}
		if (slot == 255) {		// Si no hay ningun slot libre, error
			NF_Error(6, NULL);
		}

		free(NF_BUFFER_SFX[slot]);		// Vacia el buffer
		NF_BUFFER_SFX[slot] = NULL;
		NF_SFXINFO[slot].size = 0;

		u32 medida_sfx = 0;	// Tamao del archivo a cargar

		FILE* file_sfx;		// Nombres de archivo

		char filename[256];		// Nombre del archivo

		// Abre el archivo de audio
		sprintf(filename, "%s/sfx/%s.raw", NF_ROOT_FOLDER, nombre);	// Abre el RAW
		file_sfx = fopen(filename, "rb");
		if (file_sfx) {		// Si el archivo "RAW" existe...
			// Obten el tamao del archivo
			fseek(file_sfx, 0, SEEK_END);
			medida_sfx = ftell(file_sfx);
			rewind(file_sfx);
			// Reserva el espacio en RAM
			NF_BUFFER_SFX[slot] = (char*) calloc (medida_sfx, sizeof(char));
			if (NF_BUFFER_SFX[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_BUFFER_SFX[slot], 1, medida_sfx, file_sfx);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_sfx);		// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

		NF_SFXINFO[slot].size = medida_sfx;				// Guarda el tamao del archivo
		sprintf(NF_SFXINFO[slot].name, "%s", referencia);	// Y el nombre

	}





	// Funcion NF_PlaySfx();		// Reproduce un archivo cargado previamente con NF_LoadFatSfx();
	void NF_PlaySfx(const char* referencia) {

		// Busca si existe el archivo
		u8 n;
		u8 slot = 255;
		char nombre[32];

		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar

		for (n = 0; n < NF_SFX_SLOTS; n ++) {
			if (strcmp(nombre, NF_SFXINFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_SFX_SLOTS;	// Y sal
			}
		}

		if (slot == 255) {		// Si el sonido no se encuentra, da error
			NF_Error(4, referencia); 
		}

		char* buffer;
		buffer = NULL;
		u32 buffer_size = 0;

		buffer = NF_BUFFER_SFX[slot];
		buffer_size = NF_SFXINFO[slot].size;

		NF_SFXINFO[slot].chan = AS_SoundQuickPlay(buffer);	// Reproduce el archivo y guarda el canal asignado
		
	}




	// Funcion NF_StopSfx();		// Deten un archivo reproducido previamente con NF_PlaySfx();
	void NF_StopSfx(const char* referencia) {

		// Busca si existe el archivo
		u8 n;
		u8 slot = 255;
		char nombre[32];

		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar

		for (n = 0; n < NF_SFX_SLOTS; n ++) {
			if (strcmp(nombre, NF_SFXINFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_SFX_SLOTS;	// Y sal
			}
		}

		if (slot == 255) {		// Si el sonido no se encuentra, da error
			NF_Error(4, referencia); 
		}

		AS_SoundStop(NF_SFXINFO[slot].chan);	// Deten el sonido en el canal especificado
		
	}





	// Funcion NF_UnloadSfx();		// Reproduce un archivo cargado previamente con NF_LoadFatSfx();
	void NF_UnloadSfx(const char* referencia) {

		// Busca si existe el archivo
		u8 n;
		u8 slot = 255;
		char nombre[32];

		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar

		for (n = 0; n < NF_SFX_SLOTS; n ++) {
			if (strcmp(nombre, NF_SFXINFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_SFX_SLOTS;	// Y sal
			}
		}

		if (slot == 255) {		// Si el fondo no se encuentra, da error
			NF_Error(4, referencia); 
		}

		free(NF_BUFFER_SFX[slot]);		// Vacia el buffer
		NF_BUFFER_SFX[slot] = NULL;
		sprintf(NF_SFXINFO[slot].name, "xxxNULLxxx");
		NF_SFXINFO[slot].size = 0;
		NF_SFXINFO[slot].slot = 0;		// Y marca el slot como libre

	}





	// Funcion NF_LoadCollisionMap();	// Carga un mapa de colisiones desde la FAT
	void NF_LoadCollisionMap(const char* referencia, const char* nombre, u16 mapsize) {

		// Busca un slot libre
		u8 n = 0;
		u8 slot = 255;
		for (n = 0; n < NF_CMAP_SLOTS; n ++) {
			if (NF_CMAPINFO[n].slot == 0) {	// Si esta libre, asignalo y sal
				slot = n;
				NF_CMAPINFO[n].slot = 1;
				n = NF_CMAP_SLOTS;
			}
		}
		if (slot == 255) {		// Si no hay ningun slot libre, error
			NF_Error(7, NULL);
		}

		free(NF_CMAP[slot]);		// Vacia el buffer
		NF_CMAP[slot] = NULL;

		u32 medida_colmap = 0;	// Tamao del archivo a cargar

		FILE* file_colmap;		// Nombres de archivo

		char filename[256];		// Nombre del archivo

		// Abre el archivo de mapa de colisiones
		sprintf(filename, "%s/maps/%s_Map.bin", NF_ROOT_FOLDER, nombre);	// Abre el _Map.bin
		file_colmap = fopen(filename, "rb");
		if (file_colmap) {		// Si el archivo "RAW" existe...
			// Obten el tamao del archivo
			fseek(file_colmap, 0, SEEK_END);
			medida_colmap = ftell(file_colmap);
			rewind(file_colmap);
			// Reserva el espacio en RAM
			NF_CMAP[slot] = (char*) calloc (medida_colmap, sizeof(char));
			if (NF_CMAP[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_CMAP[slot], 1, medida_colmap, file_colmap);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_colmap);	// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

		NF_CMAPINFO[slot].size = mapsize;				// Guarda el tamao del mapa
		sprintf(NF_CMAPINFO[slot].name, "%s", referencia);	// Y el nombre

	}





	// Funcion NF_CollisionMapSlot();		// Devuelve el SLOT asignado al mapa solicitado
	u8 NF_CollisionMapSlot(const char* referencia) {

		// Busca si existe el archivo
		u8 n;
		u8 slot = 255;
		char nombre[32];

		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar

		for (n = 0; n < NF_CMAP_SLOTS; n ++) {
			if (strcmp(nombre, NF_CMAPINFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_CMAP_SLOTS;	// Y sal
			}
		}

		if (slot == 255) {		// Si el mapa no se encuentra, da error
			NF_Error(4, referencia); 
		}

		return slot;	// Devuelve el SLOT del mapa solicitado

	}





	// Funcion NF_GetTile();		// Obten el "Tile" de las coordenadas dadas
	u16 NF_GetTile(u8 slot, u16 x, u16 y) {

		// Variables locales
		u16 fila = 0;
		u16 columna = 0;
		u16 tile = 0;

		fila = ((((int)(y)) >> 3) + 1);		// Calcula la fila (Pasa a entero y divide por 8)
		columna = (((int)(x)) >> 3);		// Calcula la columna

		tile = (((fila * NF_CMAPINFO[slot].size) + columna) << 1);		// Recupera el numero de TILE en el ARRAY

		return NF_CMAP[slot][tile];

	}





	// Funcion NF_UnloadCollisionMap();		// Descarga de la RAM un mapa de colisiones
	void NF_UnloadCollisionMap(const char* referencia) {

		// Busca si existe el archivo
		u8 n;
		u8 slot = 255;
		char nombre[32];

		sprintf(nombre, "%s", referencia);		// Obten el nombre a buscar

		for (n = 0; n < NF_CMAP_SLOTS; n ++) {
			if (strcmp(nombre, NF_CMAPINFO[n].name) == 0) {	// Si coincide
				slot = n;		// Guarda el SLOT donde esta
				n = NF_CMAP_SLOTS;	// Y sal
			}
		}

		if (slot == 255) {		// Si el mapa no se encuentra, da error
			NF_Error(4, referencia); 
		}

		free(NF_CMAP[slot]);				// Vacia el buffer
		NF_CMAP[slot] = NULL;
		sprintf(NF_CMAPINFO[slot].name, "xxxNULLxxx");
		NF_CMAPINFO[slot].size = 0;
		NF_CMAPINFO[slot].slot = 0;		// Y marca el slot como libre

	}





	// Funcion NF_LoadFatSprite();		// Carga un Sprite desde la FAT al Slot dado
	void NF_LoadFatSprite(u8 slot, const char* nombre){

		char filename[256];		// Nombre del archivo

		// Carga el grafico del Sprite
		free(NF_SPRITE_GFX[slot]);		// Vacia el buffer
		NF_SPRITE_GFX[slot] = NULL;

		u32 medida_sprite = 0;	// Tamao del archivo a cargar

		FILE* file_sprite;		// Nombres de archivo

		// Abre el archivo de graficos
		sprintf(filename, "%s/sprites/%s_Sprite.bin", NF_ROOT_FOLDER, nombre);	// Abre el _Sprite.bin
		file_sprite = fopen(filename, "rb");
		if (file_sprite) {		// Si el archivo "RAW" existe...
			// Obten el tamao del archivo
			fseek(file_sprite, 0, SEEK_END);
			medida_sprite = ftell(file_sprite);
			rewind(file_sprite);
			// Reserva el espacio en RAM
			NF_SPRITE_GFX[slot] = (char*) calloc (medida_sprite, sizeof(char));
			if (NF_SPRITE_GFX[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_SPRITE_GFX[slot], 1, medida_sprite, file_sprite);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_sprite);	// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

	}





	// Funcion NF_LoadFatPalette();		// Carga una paleta desde la FAT al Slot dado
	void NF_LoadFatPalette(u8 slot, const char* nombre){

		char filename[256];		// Nombre del archivo

		// Carga la paleta del Sprite
		free(NF_SPRITE_PALETTE[slot]);		// Vacia el buffer
		NF_SPRITE_PALETTE[slot] = NULL;

		u32 medida_palette = 0;	// Tamao del archivo a cargar

		FILE* file_palette;		// Nombres de archivo

		// Abre el archivo de paletas
		sprintf(filename, "%s/sprites/%s_Pal.bin", NF_ROOT_FOLDER, nombre);	// Abre el _Pal.bin
		file_palette = fopen(filename, "rb");
		if (file_palette) {		// Si el archivo "RAW" existe...
			// Obten el tamao del archivo
			fseek(file_palette, 0, SEEK_END);
			medida_palette = ftell(file_palette);
			rewind(file_palette);
			// Reserva el espacio en RAM
			NF_SPRITE_PALETTE[slot] = (char*) calloc (medida_palette, sizeof(char));
			if (NF_SPRITE_PALETTE[slot] == NULL) {
				NF_Error(2, NULL);
			}
			// Carga el archivo en RAM
			fread (NF_SPRITE_PALETTE[slot], 1, medida_palette, file_palette);
		} else {			// Si no existe...
			NF_Error(1, nombre);
		}
		fclose(file_palette);	// Cierra el archivo
		PA_WaitForVBL();		// Espera al cierre de los archivos

	}





	// Funcion NF_FreeFatSprite();		// Libera el Sprite de la RAM una vez cargado en VRAM
	void NF_FreeFatSprite(u8 slot) {
		free(NF_SPRITE_GFX[slot]);
		NF_SPRITE_GFX[slot] = NULL;
	}





	// Funcion NF_FreeFatPalette();		// Libera la paleta de la RAM una vez cargado en VRAM
	void NF_FreeFatPalette(u8 slot) {
		free(NF_SPRITE_PALETTE[slot]);
		NF_SPRITE_PALETTE[slot] = NULL;
	}





#ifdef __cplusplus
}
#endif
